@@ -1,11 +1,13 @@
 package Net::OSCAR::Common;
 
-$VERSION = 0.07;
+$VERSION = 0.08;
 
 use strict;
 use warnings;
 use vars qw(@ISA @EXPORT_OK %EXPORT_TAGS $VERSION);
 use Scalar::Util qw(dualvar);
+use Net::OSCAR::TLV;
+use Carp;
 require Exporter;
 @ISA = qw(Exporter);
 
@@ -31,6 +33,8 @@ require Exporter;
 		RATE_ALERT
 		RATE_LIMIT
 		RATE_DISCONNECT
+		GROUPPERM_OSCAR
+		GROUPPERM_AOL
 	)],
 	all => [qw(
 		ADMIN_TYPE_PASSWORD_CHANGE ADMIN_TYPE_EMAIL_CHANGE ADMIN_TYPE_SCREENNAME_FORMAT ADMIN_TYPE_ACCOUNT_CONFIRM
@@ -39,10 +43,11 @@ require Exporter;
 		FLAP_CHAN_NEWCONN FLAP_CHAN_SNAC FLAP_CHAN_ERR FLAP_CHAN_CLOSE
 		CONNTYPE_LOGIN CONNTYPE_BOS CONNTYPE_ADMIN CONNTYPE_CHAT CONNTYPE_CHATNAV
 		MODBL_ACTION_ADD MODBL_ACTION_DEL MODBL_WHAT_BUDDY MODBL_WHAT_GROUP MODBL_WHAT_PERMIT MODBL_WHAT_DENY
-		GROUP_PERMIT GROUP_DENY
+		GROUPPERM_OSCAR GROUPPERM_AOL
+		BUDTYPES
 		ENCODING
 		ERRORS
-		randchars debug_print debug_printf hexdump normalize
+		randchars debug_print debug_printf hexdump normalize tlv_decode tlv_encode tlv send_error tlvtie bltie
 	)]
 );
 @EXPORT_OK = map { @$_ } values %EXPORT_TAGS;
@@ -93,15 +98,20 @@ use constant RATE_ALERT => dualvar(2, "alert");
 use constant RATE_LIMIT => dualvar(3, "limit");
 use constant RATE_DISCONNECT => dualvar(4, "disconnect");
 
+use constant GROUPPERM_OSCAR => dualvar(0x18, "AOL Instant Messenger users");
+use constant GROUPPERM_AOL => dualvar(0x04, "AOL subscribers");
+
+use constant BUDTYPES => ("buddy", "group", "permit entry", "deny entry", "visibility/misc. data", "presence");
+
 use constant ENCODING => 'text/aolrtf; charset="us-ascii"';
 
 # I'm not 100% sure about error 29
 use constant ERRORS => split(/\n/, <<EOF);
 Invalid error
 Invalid SNAC
-Rate to host
-Rate to client
-Not logged in
+Sending too fast to host
+Sending too fast to client
+%s is not logged in, so the attempted operation (sending an IM, getting user information) was unsuccessful
 Service unavailable
 Service not defined
 Obsolete SNAC
@@ -113,7 +123,7 @@ Responses lost
 Request denied
 Busted SNAC payload
 Insufficient rights
-In local permit/deny
+%s is in your permit or deny list
 Too evil (sender)
 Too evil (receiver)
 User temporarily unavailable
@@ -219,5 +229,68 @@ sub normalize($) {
 	return lc($temp);
 }
 
+sub tlv_decode($;$) {
+	my($tlv, $tlvcnt) = @_;
+	my($type, $len, $value, %retval);
+	my $currtlv = 0;
+	my $strpos = 0;
+
+	tie %retval, "Net::OSCAR::TLV";
+
+	while(length($tlv) >= 4 and (not $tlvcnt or $currtlv < $tlvcnt)) {
+		($type, $len) = unpack("nn", $tlv);
+		$len = 0x2 if $type == 0x13;
+		$strpos += 4;
+		substr($tlv, 0, 4) = "";
+		if($len) {
+			($value) = substr($tlv, 0, $len, "");
+		} else {
+			$value = "";
+		}
+		$strpos += $len;
+		$currtlv++ unless $type == 0;
+		$retval{$type} = $value;
+	}
+
+	return $tlvcnt ? (\%retval, $strpos) : \%retval;
+}
+
+sub tlv(@) {
+	my %tlv = ();
+	tie %tlv, "Net::OSCAR::TLV";
+	while(@_) { my($key, $value) = (shift, shift); $tlv{$key} = $value; }
+	return tlv_encode(\%tlv);
+}
+
+sub tlv_encode($) {
+	my $tlv = shift;
+	my($buffer, $type, $value) = ("", 0, "");
+
+	confess "You must use a tied Net::OSCAR::TLV hash!" unless defined($tlv) and ref($tlv) eq "HASH" and defined(%$tlv) and tied(%$tlv)->isa("Net::OSCAR::TLV");
+	while (($type, $value) = each %$tlv) {
+		$value ||= "";
+		$buffer .= pack("nna*", $type, length($value), $value);
+
+	}
+	return $buffer;
+}
+
+sub send_error($$$$$;@) {
+	my($oscar, $connection, $error, $desc, $fatal, @reqdata) = @_;
+	$desc = sprintf $desc, @reqdata;
+	$oscar->callback_error($connection, $error, $desc, $fatal);
+}
+
+sub bltie(;$) {
+	my $retval = {};
+	tie %$retval, "Net::OSCAR::Buddylist", @_;
+	return $retval;
+}
+
+sub tlvtie() {
+	my $retval = {};
+	tie %$retval, "Net::OSCAR::TLV";
+	return $retval;
+}
 
 1;
